ALL_CHALLENGES = { }
ALL_MODULES = { }
MODULE_LEVELS = { }

from .challenges import Challenge, KernelChallenge, WindowsChallenge, ChallengeGroup, retry
from .register import register_challenge, register_challenges



import inspect

from . import ALL_CHALLENGES, MODULE_LEVELS, ALL_MODULES

def register_challenge(challenge):
	module = inspect.getmodule(challenge)
	module_name = module.__name__.split(".")[-1]
	challenge_name = challenge.__name__

	if challenge_name in ALL_CHALLENGES:
		print("WARNING: replacing previously-registered challenge %s with new challenge %s" % (ALL_CHALLENGES[challenge_name], challenge))

	ALL_CHALLENGES[challenge_name] = challenge
	MODULE_LEVELS.setdefault(module_name, []).append(challenge)
	ALL_MODULES[module_name] = module

def register_challenges(challenges):
	for c in challenges:
		register_challenge(c)



import traceback
import functools
import argparse
import pwnshop
import inspect
import signal
import random
import glob
import sys
import pwn #pylint:disable=import-error
import os

def challenge_class(challenge):
    if ":" not in challenge:
        assert challenge in pwnshop.ALL_CHALLENGES, "Unknown challenge specified!"
        return pwnshop.ALL_CHALLENGES[challenge]
    else:
        module, level_src = challenge.split(":")
        level = int(level_src)

        assert module in pwnshop.MODULE_LEVELS, "Uknown module specified!"
        assert 0 < level <= len(pwnshop.MODULE_LEVELS[module]), "Invalid level specified."
        return pwnshop.MODULE_LEVELS[module][level-1]

def with_challenges(f):
    @functools.wraps(f)
    def f_with_challenge(args):
        challenges = [
            challenge_class(challenge=c)(seed=args.seed, walkthrough=args.walkthrough)
            for c in args.challenges
        ]
        return f(args, challenges)
    return f_with_challenge

@with_challenges
def handle_render(args, challenges):
    for challenge in challenges:
        src = challenge.generate_source()
        if not args.lineno:
            args.out.write(src+"\n")
        else:
            for i, line in enumerate(src.splitlines()):
                args.out.write(f"{i + 1}\t{line}\n")
        if os.path.isfile(args.out.name):
            os.chmod(args.out.name, 0o644)

def handle_list(args): #pylint:disable=unused-argument
    for module,challenges in pwnshop.MODULE_LEVELS.items():
        for n,challenge in enumerate(challenges, start=1):
            print(f"{module}:{n} --- {challenge.__name__}")

@with_challenges
def handle_build(args, challenges):
    for challenge in challenges:
        binary, libs, pdb = challenge.build_binary()
        args.out.buffer.write(binary)
        if os.path.isfile(args.out.name):
            os.chmod(args.out.name, 0o755)

        if args.lpath and libs:
            os.makedirs(args.lpath, exist_ok=True)
            for lib_name, lib_bytes in libs:
                lib_path = args.lpath + f'/{lib_name}'
                with open(lib_path, 'wb+') as f:
                    f.write(lib_bytes)
                os.chmod(lib_path, 0o755)
        if pdb:
            with open(f"{args.out.name.replace('.exe', '.pdb')}", 'wb') as f:
                f.write(pdb)

class TimeoutError(Exception):
    pass
def raise_timeout(signum, stack):
    raise TimeoutError("ACTION TIMED OUT")

def verify_challenge(challenge, debug=False, flag=None, strace=False):
    if debug:
        pwn.context.log_level = "DEBUG"

    if flag:
        with open("/flag", "wb") as f:
            f.write(flag.encode())

    if "strace" in inspect.getfullargspec(challenge.verify)[0]:
        challenge.verify(strace=strace)
    else:
        challenge.verify()

def verify_many(args, challenges):
    failures = [ ]
    for challenge in challenges:
        name = challenge.__name__ if type(challenge) is type else type(challenge).__name__

        print(f"VERIFYING: {name}")

        try:
            if type(challenge) is type:
                challenge = challenge(seed=args.seed, walkthrough=args.walkthrough)

            if args.timeout:
                signal.signal(signal.SIGALRM, raise_timeout)
                signal.alarm(args.timeout)

            verify_challenge(challenge, debug=args.debug, flag=args.flag, strace=args.strace)
            signal.alarm(0)
            print(f"SUCCEEDED: {name}")
        except NotImplementedError:
            failures.append(challenge)
            print(f"MISSING: {name}")
        except Exception: #pylint:disable=broad-exception-caught
            print(traceback.format_exc())
            failures.append(challenge)
            print(f"FAILED: {name}")

    return False if failures else True

@with_challenges
def handle_verify(args, challenges):
    return verify_many(args, challenges)

def handle_verify_module(args):
    return verify_many(args, pwnshop.MODULE_LEVELS[args.module])

def handle_verify_all(args):
    return verify_many(args, pwnshop.ALL_CHALLENGES.values())


def main():
    parser = argparse.ArgumentParser(description="pwnshop challenge emitter", formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument(
        "-C",
        "--challenge-location",
        required=False,
        help="a path glob to import challenges from (either /path/to/module.py or /path/to/package/ or /some/glob/*, but avoid shell-expansion for the latter!). Defaults to cwd.",
        default=os.getcwd()
    )
    commands = parser.add_subparsers(help="the action for pwnshop to perform", required=True, dest="ACTION")
    command_render = commands.add_parser("list", help="list known challenges")
    command_render = commands.add_parser("render", help="render the source code of a challenge")
    command_build = commands.add_parser("build", help="build the binary code of a challenge")
    command_verify = commands.add_parser("verify", help="verify the functionality of a challenge")
    command_verify_module = commands.add_parser("verify-module", help="verify the functionality of all challenges in a module")
    command_verify_all = commands.add_parser("verify-all", help="verify the functionality of all challenges")

    command_render.add_argument(
        "-l",
        "--lineno",
        action="store_true",
        help="render line numbers",
    )

    command_build.add_argument(
        "--lpath",
        help="Location to store needed library files",
    )

    for c in [ command_verify, command_verify_module, command_verify_all ]:
        c.add_argument(
            "-t",
            "--strace",
            action="store_true",
            help="print out strace information during verification",
        )

        c.add_argument(
            "-f",
            "--flag",
            help="change the flag to be verified against",
        )

        c.add_argument(
            "-T",
            "--timeout",
            type=int,
            help="set a per-challenge timeout (in seconds) for verification",
        )


    # where to write
    for c in [ command_render, command_build ]:
        c.add_argument(
            "-O",
            "--out",
            type=argparse.FileType('w'),
            default='-',
            help="change the output destination"
        )

    # common arguments
    for subparser in commands.choices.values():
        subparser.add_argument(
            "-s",
            "--seed",
            required=False,
            default=random.randrange(0, 999999),
            help="the seed from which to generate the challenge (default: random)",
        )

        subparser.add_argument(
            "-d",
            "--debug",
            action="store_true",
            help="print out debugging information",
        )

        subparser.add_argument(
            "-w",
            "--walkthrough",
            action="store_true",
            help="enable challenge walkthrough mode",
        )

    # common to all single-challenge commands
    for subparser in [ command_render, command_build, command_verify ]:
        subparser.add_argument("challenges", help="the challenges, as multiple ChallengeClassName or ModuleName:level_number", nargs="+")

    command_verify_module.add_argument("module", help="the module to verify")

    parser.epilog = f"""Commands usage:\n\t{command_render.format_usage()}\t{command_build.format_usage()}\t{command_verify.format_usage()}\t{command_verify_module.format_usage()}\t{command_verify_all.format_usage()}"""

    args = parser.parse_args()
    pwn.context.log_level = "ERROR"

    import_dir = args.challenge_location
    if not os.path.isdir(import_dir):
        print(f"Error, {import_dir=} does not exist")
        return -1
    imports = glob.glob(import_dir)
    for i in imports:
        i = i.rstrip("/")
        sys.path.append(os.path.realpath(os.path.dirname(i)))
        try:
            module_name = os.path.basename(i).split(".py")[0]
            __import__(module_name)
        finally:
            sys.path.pop()

    r = globals()["handle_" + args.ACTION.replace('-', '_')](args)
    if r in (None, True):
        return 0
    elif type(r) is int:
        return r
    else:
        return 1

if __name__ == "__main__":
    sys.exit(main())



import sys
import os
import re
import string
import random
import signal
import inspect
import subprocess
import tempfile
import textwrap
import contextlib
import traceback

import docker
import pyastyle
import pwn
from jinja2 import Environment, PackageLoader, ChoiceLoader, contextfilter

from ..register import register_challenge

pwn.context.arch = "x86_64"
pwn.context.encoding = "latin"

def hex_str_repr(s):
    hex_s = s.encode("latin").hex()
    return "".join("\\x" + hex_s[i : i + 2] for i in range(0, len(hex_s), 2))


def layout_text(text):
    lines = textwrap.wrap(textwrap.dedent(text), width=120)
    if lines:
        lines[-1] += "\\n"
    else:
        lines = [""]
    return "\n".join(f'puts("{line}");' for line in lines)


@contextfilter
def layout_text_walkthrough(context, text):
    if not context.get("walkthrough"):
        return "\n"
    return layout_text(text)


class Challenge:
    COMPILER = "gcc"
    PIE = None
    RELRO = "full"
    MASM_FLAG = "-masm=intel"
    OPTIMIZATION_FLAG = "-O0"
    CANARY = None
    FRAME_POINTER = None
    STATIC = False
    EXEC_STACK = False
    STRIP = False
    DEBUG_SYMBOLS = False
    LINK_LIBRARIES = []

    build_image = None

    context = {
        "min": min,
        "max": max,
        "hex": hex,
        "hex_str_repr": hex_str_repr,
        "layout_text": layout_text,
        "layout_text_walkthrough": layout_text,
    }

    def __init__(self, *, seed, **kwargs):
        self.seed = seed
        self.random = random.Random(seed)
        self.walkthrough = kwargs.get("walkthrough")

    def __init_subclass__(cls, register=True):
        cls_module = inspect.getmodule(cls)
        if register and getattr(cls_module, "PWNSHOP_AUTOREGISTER", True):
            register_challenge(cls)

    @property
    def TEMPLATE_PATH(self):
        raise NotImplementedError()

    @property
    def local_context(self):
        return {
            e: getattr(self, e)
            for e in dir(self)
            if not e.startswith("_") and e == e.upper()
        }

    @property
    def flag(self):
        with open("/flag", "rb") as f:
            return f.read()

    def random_word(self, length, vocabulary=string.ascii_lowercase):
        return "".join(self.random.choice(vocabulary) for _ in range(length))

    def generate_source(self):
        env = Environment(loader=ChoiceLoader([
            PackageLoader(__name__, ""),
            PackageLoader(inspect.getmodule(self).__name__, ""),
            PackageLoader(inspect.getmodule(self).__name__, ".."),
        ]), trim_blocks=True)
        env.filters["layout_text"] = layout_text
        env.filters["layout_text_walkthrough"] = layout_text_walkthrough
        template = env.get_template(self.TEMPLATE_PATH)
        result = template.render(
            challenge=self,
            walkthrough=self.walkthrough,
            **self.context,
            **self.local_context,
        )
        result = pyastyle.format(result, "--style=allman")
        result = re.sub("\n{2,}", "\n\n", result)
        return result

    def build_compiler_cmd(self):
        cmd = [self.COMPILER]

        if self.RELRO == "full":
            cmd.append("-Wl,-z,relro,-z,now")
        elif self.RELRO  == "partial":
            cmd.append("-Wl,-z,relro,-z,lazy")
        elif self.RELRO == "none":
            cmd.append("-Wl,-z,norelro")

        if self.PIE is True:
            cmd.append("-pie")
        elif self.PIE is False:
            cmd.append("-no-pie")

        if self.CANARY is True:
            cmd.append("-fstack-protector")
        elif self.CANARY is False:
            cmd.append("-fno-stack-protector")

        if self.FRAME_POINTER is True:
            cmd.append("-fno-omit-frame-pointer")
        elif self.FRAME_POINTER is False:
            cmd.append("-fomit-frame-pointer")

        if self.STATIC and self.PIE:
            cmd.append("-static-pie")
        elif self.STATIC:
            cmd.append("-static")

        if self.EXEC_STACK:
            cmd.append("-z")
            cmd.append("execstack")

        if self.DEBUG_SYMBOLS:
            cmd.append("-g")

        if self.STRIP:
            cmd.append("-s")

        if self.MASM_FLAG:
            cmd.append(self.MASM_FLAG)
        if self.OPTIMIZATION_FLAG:
            cmd.append(self.OPTIMIZATION_FLAG)

        cmd.append("-w")

        cmd.append("-x")
        cmd.append("c")

        cmd.append("-")

        for lib in self.LINK_LIBRARIES:
            cmd.append("-l" + lib)

        return cmd

    def build_binary(self, source=None):
        if not source:
            source = self.generate_source()

        cmd = self.build_compiler_cmd()

        if self.build_image is None:
            with tempfile.TemporaryDirectory(prefix='pwnshop-') as workdir:
                bin_path = f"{workdir}/{self.__class__.__name__.lower()}"
                cmd.append("-o")
                cmd.append(bin_path)

                subprocess.check_output(cmd, input=source.encode())
                with open(bin_path, 'rb') as f:
                    binary = f.read()
                libs = None
        else:
            binary, libs = self._containerized_build(cmd, source)

        return binary, libs, None

    @contextlib.contextmanager
    def setup_environment(self, binary=None, *, path=None, flag_symlink=None):
        work_dir = tempfile.mkdtemp(prefix='pwnshop-')
        libs = None

        if not binary:
            binary, libs, _ = self.build_binary()
        if not path:
            os.makedirs(work_dir, exist_ok=True)
            path = work_dir + "/" + self.__class__.__name__.lower()
            if isinstance(self, KernelChallenge):
                path += ".ko"

        with open(path, "wb") as f:
            f.write(binary)
        os.chmod(path, 0o4755)

        if libs:
            lib_dir = work_dir + '/lib'
            os.makedirs(lib_dir, exist_ok=True)
            for lib_name, lib_bytes in libs:
                lib_file = lib_dir + '/' + lib_name
                with open(lib_file, 'wb') as f:
                    f.write(lib_bytes)
                os.chmod(lib_file, 0o0755)

        if flag_symlink:
            os.symlink("/flag", f"{flag_symlink}")

        yield path

    @contextlib.contextmanager
    def verify(self, binary=None, cmd_args=None, argv=None, **kwargs):
        raise NotImplementedError()

    @contextlib.contextmanager
    def run_challenge(
        self,
        binary=None,
        cmd_args=None,
        argv=None,
        *,
        executable_path=None,
        flag_symlink=None,
        close_stdin=False,
        strace=False,
        **kwargs,
    ):
        environment_ctx = None
        if not executable_path:
            environment_ctx = self.setup_environment(
                binary=binary, flag_symlink=flag_symlink
            )
            executable_path = environment_ctx.__enter__()

        with open("/flag", "rb") as f:
            assert f.read() == self.flag

        if argv is None:
            argv = [executable_path]
            if cmd_args:
                argv += cmd_args
            if strace:
                argv = ["strace"] + argv
                executable_path = "strace"
        else:
            assert not strace

        cwd = os.path.dirname(executable_path)

        with pwn.process(
            argv, executable=executable_path, cwd="/", **kwargs
        ) as process:
            if close_stdin:
                process.stdin.close()
            try:
                yield process
            finally:
                if environment_ctx:
                    environment_ctx.__exit__(*sys.exc_info())

    def run_sh(self, command, **kwargs):
        return pwn.process(command, shell=True, **kwargs)

    def metadata(self):
        """
        Returns a dictionary to be yaml'ed for the dojo.
        """
        return {
            "class_name": self.__class__.__name__,
            "description": inspect.cleandoc(self.__doc__) if self.__doc__ else ""
        }

    def _containerized_build(self, cmd, source):
        """
        Spins up a docker container to build target, returns binary and linked libraries
        """
        cont_vpath =  '/mnt/pwnshop'
        bin_path =  cont_vpath + '/' + self.__class__.__name__.lower()

        cmd.append("-o")
        cmd.append(bin_path)
        cmd.append(f'{cont_vpath}/source.c')

        with tempfile.TemporaryDirectory(prefix='pwnshop-') as workdir:
            os.makedirs(f"{workdir}/lib", exist_ok=True)
            client = docker.from_env()
            img, tag = self.build_image.split(':')
            client.images.pull(img, tag=tag)

            #TODO: container life is context manager
            container = client.containers.run(img + ':' + tag,
                                    'sleep 300',
                                    auto_remove=True,
                                    detach=True,
                                    volumes = {workdir : {'bind': cont_vpath,
                                                                'mode': 'rw'}})
            code, out = container.exec_run('/bin/bash -c "apt update && apt install -y gcc patchelf && mkdir -p /tmp/pwnshop"')
            with open(f'{workdir}/source.c', 'w+') as f:
                f.write(source)

            ret, out = container.exec_run(cmd)
            #container.exec_run(f'chmod 0777 ' + bin_path)
            #container.exec_run(f'chown {os.getuid()}:{os.getgid()} ' + bin_path)
            assert ret == 0, out

            ret, out = container.exec_run("ldd " + bin_path)
            assert ret == 0
            lib_paths = filter(lambda x: '/' in x, out.decode().split())

            libs = []
            for p in lib_paths:
                lib_name = os.path.basename(p)
                container.exec_run(f'cp {p} {cont_vpath}/lib/{lib_name}')

                container.exec_run(f'chmod 0766 {cont_vpath}/lib/{lib_name}')
                container.exec_run(f'chown {os.getuid()}:{os.getgid()} {cont_vpath}/lib/{lib_name}')

                with open(f'{workdir}/lib/{lib_name}', 'rb') as f:
                    libs.append((lib_name, f.read()))
                if "ld-linux" in lib_name:
                    ret, out = container.exec_run(f'patchelf --set-interpreter /challenge/lib/{lib_name} ' + bin_path, workdir=cont_vpath)
                else:
                    container.exec_run(f'patchelf --replace-needed {lib_name} /challenge/lib/{lib_name} ' + bin_path, workdir=cont_vpath)

            with open(f"{workdir}/{self.__class__.__name__.lower()}", 'rb') as f:
                binary = f.read()

            return binary, libs

class WindowsChallenge(Challenge, register=False):
    COMPILER = "cl"
    CANARY = None
    FRAME_POINTER = True
    PDB = True
    CFG = True
    DYNAMIC_BASE = True
    ASLR_HIGH_ENTROPY = True
    MAKE_DLL = False

    def build_compiler_cmd(self):
        cmd = [self.COMPILER]

        if self.CANARY is True:
            cmd.append("/GS")
        else:
            cmd.append("/GS-")

        if self.FRAME_POINTER is False:
            cmd.append("/0y")

        if self.PDB:
            cmd.append("/Zi")

        if self.CFG:
            cmd.append("/guard:cf")
        else:
            cmd.append("/guard:cf-")

        if self.MAKE_DLL:
            cmd.append("/LD")

        return cmd
        # Linker options
        cmd.append('/LINK')
        if self.DYNAMIC_BASE is True:
            cmd.append('/DYNAMICBASE')
        else:
            cmd.append('/DYNAMICBASE:NO')

        if self.ASLR_HIGH_ENTROPY is True:
            cmd.append('/HIGHENTROPYVA')

        for lib in self.LINK_LIBRARIES:
            cmd.append(lib)

        return cmd

    def build_binary(self, source=None):
        if not source:
            source = self.generate_source()

        cmd = self.build_compiler_cmd()

        if self.build_image is None:
            with tempfile.TemporaryDirectory(prefix='pwnshop-') as workdir:
                src_path = f"{workdir}/{self.__class__.__name__.lower()}.c"

                if self.MAKE_DLL:
                    bin_path = f"{workdir}/{self.__class__.__name__.lower()}.dll"
                else:
                    bin_path = f"{workdir}/{self.__class__.__name__.lower()}.exe"
                pdb_path = f"{workdir}/{self.__class__.__name__.lower()}.pdb"
                with open(src_path, 'w') as f:
                    f.write(source)

                cmd.append(src_path)
                subprocess.check_output(cmd, cwd=workdir)
                with open(bin_path, 'rb') as f:
                    binary = f.read()

                if self.PDB:
                    with open(pdb_path, 'rb') as f:
                        pdb = f.read()
                else:
                    pdb = None
                return binary, None, pdb

        else:
            raise NotImplementedError("Containerized Windows build not supported")

class KernelChallenge(Challenge, register=False):
    def build_binary(self, source=None):
        with tempfile.TemporaryDirectory() as workdir:
            with open(f"{workdir}/Makefile", "w") as f:
                f.write(
                    textwrap.dedent(
                        f"""
                        obj-m += challenge.o

                        all:
                        \tmake -C /opt/linux/linux-5.4 M={workdir} modules
                        clean:
                        \tmake -C /opt/linux/linux-5.4 M={workdir} clean
                        """
                    )
                )

            cmd = ["make", "-C", workdir]

            if not source:
                source = self.generate_source()

            with open(f"{workdir}/challenge.c", "w") as f:
                f.write(source)

            subprocess.run(cmd, stdout=sys.stderr)

            with open(f"{workdir}/challenge.ko", "rb") as f:
                binary = f.read()

            return binary, None, None

    @contextlib.contextmanager
    def run_challenge(
        self,
        binary=None,
        *,
        executable_path=None,
        flag_symlink=None,
        **kwargs,
    ):
        environment_ctx = None
        if not executable_path:
            environment_ctx = self.setup_environment(
                binary=binary, flag_symlink=flag_symlink
            )
            executable_path = environment_ctx.__enter__()

        # ./run.sh ./generate.py -m BabyKernel -i1 -v -l1 -vvv
        subprocess.run(["passwd", "-d", "root"])
        subprocess.run(["vm", "restart"])
        subprocess.run(["ln", "-sf", executable_path, "/challenge/challenge.ko"])

        try:
            yield
        finally:
            if environment_ctx:
                environment_ctx.__exit__(*sys.exc_info())

    def run_sh(self, command, **kwargs):
        return pwn.process(["vm", "exec", command], **kwargs)

    def run_c(self, src, *, user=None, flags=[]):
        with open("/tmp/program.c", "w") as f:
            f.write(textwrap.dedent(src))
        subprocess.run(
            ["gcc", "-static", "/tmp/program.c", "-o", "/tmp/program"] + flags
        )
        command = "/tmp/program"
        if user:
            command = f"su {user} -c {command}"
        return self.run_sh(command)

    def symbol_address(self, symbol):
        data = self.run_sh(f"grep -P '^[0-9a-f]+\\ .\\ {symbol}(\\t|$)' /proc/kallsyms | grep -oP '^[0-9a-f]+'").readall().strip()
        assert len(data.split()) == 1
        return int(data, 16)


class ChallengeGroup(Challenge, register=False):
    challenges = NotImplemented

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.challenge_instances = [
            challenge(*args, **kwargs) for challenge in self.challenges
        ]

    def generate_source(self):
        for challenge in self.challenge_instances:
            yield challenge.generate_source()

    def build_binary(self):
        for challenge in self.challenge_instances:
            yield challenge.build_binary()

    @contextlib.contextmanager
    def setup_environment(self, binary=None, *, path=None, flag_symlink=None):
        environment_ctxs = [
            challenge.setup_environment() for challenge in self.challenge_instances
        ]
        paths = [ctx.__enter__() for ctx in environment_ctxs]
        try:
            yield paths[0]
        finally:
            for ctx in environment_ctxs:
                ctx.__exit__(*sys.exc_info())

    def run_challenge(self, binary=None, *, executable_path=None, **kwargs):
        environment_ctx = None
        if not executable_path:
            environment_ctx = self.setup_environment(binary=binary)
            executable_path = environment_ctx.__enter__()

        challenge = self.challenge_instances[0]
        result = challenge.run_challenge(binary=binary, executable_path=executable_path)

        if environment_ctx:
            environment_ctx.__exit__(*sys.exc_info())

        return result

def retry(max_attempts, timeout=None):
    def wrapper(func):
        @contextlib.wraps(func)
        def wrapped(*args, **kwargs):
            for _ in range(max_attempts):
                try:
                    if timeout:
                        def alarm(*args):
                            raise AssertionError("ATTEMPT TIMED OUT")
                        signal.signal(signal.SIGALRM, alarm)
                        signal.alarm(timeout)
                    return func(*args, **kwargs)
                except AssertionError:
                    traceback.print_exc()
                finally:
                    signal.alarm(0)
            else:
                raise Exception(f"Failed after {max_attempts} attempts!")

        return wrapped

    return wrapper



import os

from setuptools import setup
from setuptools import find_packages

packages = find_packages(exclude=["example_module"]) + [ "pwnshop.challenges.base_templates", "pwnshop.challenges.base" ]

setup(
    name="pwnshop",
    version="3.5",
    python_requires=">=3.8",
    packages=packages,
    install_requires=["jinja2==3.0.3", "nbconvert==6.4.4", "asteval", "pyastyle", "pwntools", "ezmp", "docker"],
    package_data={ dd.replace("/","."):cc for dd,cc in ( (d,[f for f in c if f.endswith(".c")]) for d,_,c in os.walk("pwnshop") ) if cc },
    description="A framework for generating CTF challenges for learning",
    url="https://github.com/pwncollege/pwnshop",
    entry_points={
        "console_scripts": [
            "pwnshop = pwnshop.__main__:main"
        ],
    },
)
